package com.ruoyi.system.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.common.utils.ip.IpUtils;
import com.ruoyi.common.utils.wechat.WechatKeys;
import com.ruoyi.system.domain.TPatient;
import com.ruoyi.system.domain.TWechatUser;
import com.ruoyi.system.domain.req.GetOpenidReq;
import com.ruoyi.system.domain.req.LoginForMpReq;
import com.ruoyi.system.domain.resp.GetOpenidResp;
import com.ruoyi.system.mapper.TPatientMapper;
import com.ruoyi.system.mapper.TWechatUserMapper;
import com.ruoyi.system.service.IWechatService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.Objects;

@Service
public class WechatServiceImpl implements IWechatService {
    private static final Logger logger = LoggerFactory.getLogger(WechatServiceImpl.class);
    /**
     * 微信小程序AppID
     */
    @Value("${wx.mpAppId}")
    private String appId;
    /**
     * 微信小程序AppSecret
     */
    @Value("${wx.mpSecret}")
    private String appSecret;

    @Resource
    private TWechatUserMapper tWechatUserMapper;

    @Resource
    private TPatientMapper tPatientMapper;

    @Resource
    private UserTokenService userTokenService;

    @Override
    public GetOpenidResp getOpenid(GetOpenidReq req, HttpServletRequest request) {
        //查询openid
        String openid = queryOpenid(req);
        TWechatUser wechatUser = updateWechatUser(openid, req, request);
        GetOpenidResp resp = new GetOpenidResp();
        if (Objects.isNull(wechatUser.getUserId())) {
            resp.setBindFlag(false);
        } else {
            // 已绑定信息 直接登录
            resp.setBindFlag(true);
            TPatient tPatient = tPatientMapper.selectTPatientById(wechatUser.getUserId());
            String token = buildTokenByPatient(tPatient);
            resp.setToken(token);
        }
        resp.setOpenid(openid);
        return resp;
    }

    @Override
    public String loginForMp(LoginForMpReq req) {
        // 查询手机号是否存在患者信息
        TPatient tPatient = new TPatient();
        BeanUtils.copyProperties(req, tPatient);
        TPatient currentPatient = tPatientMapper.queryTPatient(tPatient);
        TWechatUser wechatUser = new TWechatUser();
        if (Objects.nonNull(currentPatient)) {
            // 存在 需要把患者信息更新 并绑定openid
            BeanUtils.copyProperties(req, currentPatient);
            tPatientMapper.updateTPatient(currentPatient);
            wechatUser.setUserId(currentPatient.getId());
            tPatient.setId(currentPatient.getId());
        } else {
            // 不存在 患者信息入库
            tPatient.setDelFlag("0");
            tPatient.setCreateTime(new Date());
            tPatient.setUpdateTime(new Date());
            tPatientMapper.insertTPatient(tPatient);
            wechatUser.setUserId(tPatient.getId());
        }
        // 绑定openid
        wechatUser.setOpenid(req.getOpenId());
        tWechatUserMapper.updateWechatUserByOpenId(wechatUser);
        // 登录获取token
        String token = buildTokenByPatient(tPatient);
        return token;
    }

    private String queryOpenid(GetOpenidReq req){
        String url = String.format(WechatKeys.MP_CODE_TO_OPENID_URL, appId, appSecret, req.getCode());
        String json = HttpUtils.sendGet(url);
        JSONObject jsonObject = JSONObject.parseObject(json);
        String session_key = (String) jsonObject.get("session_key");
        String openid = (String) jsonObject.get("openid");
        logger.info("login openid:{},session_key:{}", openid, session_key);
        if (StringUtils.isEmpty(openid)) {
            throw new ServiceException("未获取到openid");
        }
        return openid;
    }

    private TWechatUser updateWechatUser(String openid, GetOpenidReq req, HttpServletRequest request){
        //查询当前微信用户是否存在
        TWechatUser wechatUser = tWechatUserMapper.selectWechatUserByOpenid(openid);
        if (wechatUser == null){
            wechatUser = new TWechatUser();
            wechatUser.setOpenid(openid);
            wechatUser.setAvatar(req.getAvatarUrl());
            wechatUser.setNickname(req.getNickName());
            wechatUser.setLastLoginTime(new Date());
            wechatUser.setLastLoginIp(IpUtils.getIpAddr(request));
            wechatUser.setStatus((byte) 0);
            tWechatUserMapper.insert(wechatUser);

        }
        wechatUser.setLastLoginTime(new Date());
        wechatUser.setLastLoginIp(IpUtils.getIpAddr(request));
        tWechatUserMapper.updateByPrimaryKeySelective(wechatUser);
        return wechatUser;
    }

    private String buildTokenByPatient(TPatient tPatient) {
        if (Objects.isNull(tPatient)) {
            return StringUtils.EMPTY;
        }
        LoginUser loginUser = new LoginUser();
        loginUser.setUserId(tPatient.getId());
        SysUser user = new SysUser();
        user.setUserId(tPatient.getId());
        user.setDelFlag(tPatient.getDelFlag());
        user.setSex(tPatient.getSex());
        user.setPhonenumber(tPatient.getPhone());
        loginUser.setUser(user);
        String token = userTokenService.createToken(loginUser);
        return token;
    }
}
